
InstallMethod( ModularGroupAlgebra, [IsPcGroup], function(G)
    if not IsPGroup(G) then TryNextMethod(); fi;
    return GroupRing(GF(PrimePGroup(G)), G);
end );

CanoFormWithAutGroupOfAugIdeal := function(A)
    return CanoFormWithAutGroupOfTable(TableByPowerBasisOfAug(A));
end;

CanoFormOfAugIdeal := function(A)
    return CanoFormWithAutGroupOfAugIdeal(A).cano;
end;

AutomorphismGroupOfAugIdeal := function(A)
    local G, d, F, a, B;
    G := CanoFormWithAutGroupOfAugIdeal(A);
    d := Length(G.one);
    F := G.field;
    a := Concatenation(G.glAutos, G.agAutos);
    B := Subgroup(GL(d, F), a);
    SetSize(B, G.size);
    return B;
end;

MatPlus := function( mat, F, k )
    local M, i, j;
    M := MutableNullMat( Length(mat)+1, Length(mat)+1, F );
    M[1][k] := One(F);
    for i in [1..Length(mat)] do
        for j in [1..Length(mat)] do
            M[i+1][j+1] := mat[i][j];
        od;
    od;
    return M;
end;

InstallMethod( CanonicalForm, [IsAlgebra and IsGroupRing], function(A)
    local T, F, n, i;
    T := CanoFormWithAutGroupOfAugIdeal(A);
    F := T.field;
    T := T.cano;
    n := Length(T);
    for i in [1..n] do T[i] := MatPlus( T[i], F, i+1); od;
    return Concatenation( [IdentityMat(n+1,F)], T);
end);

InstallMethod( AutomorphismGroup, [IsAlgebra and IsGroupRing], function(A)
    local G, H, U, B, b, c, inv, autos, aut, i;

    # compute cano form
    G := CanoFormWithAutGroupOfAugIdeal(A);

    # get base change
    B := CoeffsPowerBasisOfAug(A);
    b := Concatenation([Coefficients(Basis(A),One(A))], B.basis);
    c := b^-1;

    # get inverse of isom
    inv := G.isom^-1;

    # set up 
    autos := [];

    # fill in autos
    Info( InfoModIsom, 1, "translate autos");
    for i in [1..Length(G.glAutos)] do
        aut := inv * G.glAutos[i] * G.isom;
        aut := MatPlus( aut, G.field, 1 );
        aut := b * aut * c;
        autos[i] := aut;
    od;
    for i in [1..Length(G.agAutos)] do
        aut := inv * G.agAutos[i] * G.isom;
        aut := MatPlus( aut, G.field, 1 );
        aut := b * aut * c;
        autos[i] := aut;
    od;

    U := Subgroup(GL(Length(G.one)+1, G.field), autos);
    SetSize(U, G.size);

    # that's it
    return U;
end );

