
InitCanoForm := function( T )
    local F, d, G, R;

    # set up
    F := T.field;
    d := T.rank;
    #Print("Layer ",1," of dimension ",d,"\n");

    # adjust global variable
    if d = 2 and Size(F) <= 4 then 
        USE_PARTI := false; 
    else
        USE_PARTI := true;
    fi;

    # init aut group
    G := InitAutomGroup( T );

    # init cano form
    R := rec( tab := List( [1..d], x -> NullMat(d,d,F) ),
              dfR := List( [1..d], x -> 0 ),
              wgR := List( [1..d], x -> 1 ),
              iso := G.basis);

    # add info 
    T.cano := R;
    T.auto := G;
end;

ExtendCanoForm := function( Tab, i )
    local F, T, w, d, R, C, S, U, W, Q, G;

    # catch some arguments from  A
    F := Tab.field;
    T := Tab.table;
    w := Tab.weights;
    d := Length(Filtered(w, x -> x <= i));
    #Print("Layer ",i," of dimension ",d,"\n");

    # catch some arguments from G
    R := Tab.cano;
    G := Tab.auto;

    # compute cover of T
    S := T{[1..d]}{[1..d]}{[1..d]};
    C := CoveringTable( R );
    AddDefsAndIsom( C, R, S );
    U := AllowableSubspace( C );

    # report status
    #Print("   got dim(M) = ",Length(C.mul),
    #            " dim(N) = ",C.nuc,
    #            " dim(U) = ",Length(U),"\n");  

    # lift autos
    InduceAutosToMult( G, C, R );

    # stabilize
    W := HybridMatrixCanoForm( G, U );
    if IsInt(W) then return W; fi;

    # extend quotient
    Q := QuotientTable( C, R, W, w{[1..d]} );
    AddIsomQuotientTable( Q, S, C, W );

    # induce autos
    InduceAutosToQuot( G, Q );

    # report
    #Print("   got group of size ",G.glOrder," * ");
    #Print(Characteristic(G.field),"^",Length(G.agAutos),"\n");   

    # add info 
    Tab.cano := Q;
    Tab.auto := G;
end;

#############################################################################
##
#F RefineBins( bins, vals )
##
RefineBinByVals := function( bin, vals )
    return List(Set(vals{bin}), x -> Filtered(bin, y -> vals[y]=x));
end;
    
RefineBinsByVals := function( bins, vals )
    local refs;
    refs := Concatenation(List(bins, x -> RefineBinByVals(x, vals )));
    return Filtered(refs, x -> Length(x)>1);
end;

#############################################################################
##
#F CheckBin(p, n, k, list)
##
CheckBin := function(p, n, k, list)
    local grps, algs, tabs, bins, l, m, vals, todo, i, j;

    l := Length(list);

    # set up
    grps := List(list, x -> SmallGroup(p^n, x));
    algs := List(grps, ModularGroupAlgebra);
    tabs := [];
    Print("compute tables through power series \n");
    for i in [1..l] do
        tabs[i] := ShallowCopy(TableByPowerBasisOfAug(algs[i]));
        Print("  determined table for ",i,"\n");
    od;
    Print("\n");

    # init bins
    bins := [[1..l]];
    Print("refine bin \n");

    # refine by weights
    vals := List( tabs, x -> x.weights );
    bins := RefineBinsByVals( bins, vals );
    Print("  weights yields bins ",bins,"\n");
    todo := Flat(bins);
    if Length(todo) = 0 then return; fi;

    # start up and refine by aut group
    vals := [];
    for i in [1..l] do
        if i in todo then 
            InitCanoForm(tabs[i]); 
            vals[i] := [tabs[i].auto.size, tabs[i].auto.partition];
        fi;
    od;
    bins := RefineBinsByVals( bins, vals );
    Print("  layer 1 yields bins ",bins,"\n");
    todo := Flat(bins);
    if Length(todo) = 0 then return; fi;

    # get limit
    m := Maximum( List(tabs, x -> Maximum(x.weights) ) );
    if not IsBool(k) then m := Minimum(m,k); fi;

    # loop and refine by canonical forms
    for j in [2..m] do
        vals := [];
        for i in [1..l] do
            if i in todo then 
                ExtendCanoForm(tabs[i], j); 
                vals[i] := tabs[i].cano;
            fi;
        od;
        bins := RefineBinsByVals( bins, vals );
        Print("  layer ",j," yields bins ",bins,"\n");
        todo := Flat(bins);
        if Length(todo) = 0 then return; fi;
    od;
 
    return bins;
end;

