
CanoFormWithAutGroupOfTable := function(Tab)
    local F, B, G, R, S, T, C, M, U, W, Q, w, d, l, i, s, n, p;

    # catch some arguments
    F := Tab.field;
    p := Characteristic(F);
    T := Tab.table;
    w := Tab.weights;
    d := Tab.rank;
    l := Length(Set(w));

    # do the first step I/I^2
    Info( InfoModIsom, 1, "layer 1 has dimension ",d);
    G := InitAutomGroup(Tab);
    R := rec( tab := List( [1..d], x -> NullMat(d,d,F) ),
              dfR := List( [1..d], x -> 0 ),
              iso := G.basis,
              wgR := w{[1..d]} );
    Unbind(G.basis);

    # loop over other steps I/I^i
    for i in [2..l] do
 
        # catch the layer
        d := Length( Filtered( w, x -> x <= i ) );
        S := T{[1..d]}{[1..d]}{[1..d]};

        # report on current state
        Info( InfoModIsom, 1, 
                 "induce to layer ",i,
                 " of dim ",d,
                 " using aut group of order ",G.glOrder, 
                 " * ", p,"^",Length(G.agAutos) );

        # compute cover and allowable subspace
        C := CoveringTable( R );
        AddDefsAndIsom( C, R, S );
        if CHECK_COV and not CheckCoverEpimorphism(C, S) then 
            Error("wrong cover-epi");
        fi;
        U := AllowableSubspace( C );

        # report result
        Info( InfoModIsom, 1, 
                "   got dim(M) = ",Length(C.mul), 
                " and dim(U) = ",Length(U) );

        # lift autos
        InduceAutosToMult( G, C, R );

        # stabilize and pull out at too long orbit
        W := HybridMatrixCanoForm( G, U );
        if IsInt(W) then return fail; fi;

        # extend quotient
        Q := QuotientTable( C, R, W, w{[1..d]} );
        AddIsomQuotientTable( Q, S, C, W );

        # induce autos
        InduceAutosToQuot( G, Q );

        # check
        if CHECK_AUT then 
            if not CheckGroupByTable( G, Q.tab ) then 
                Error("autos wrong");
            elif not CheckIsomByTables( Q.tab, S, Q.iso ) then 
                Error("isom wrong");
            fi;
            Print("   checked \n");
        fi;
        
        # reset
        R := Q;
    od;

    # finally add result to group
    G.cano := Q.tab;
    G.isom := Q.iso;
    G.wgts := w;
    return G; 
end; 

CanoFormWithAutGroupOfNilpotentAlgebra := function(A)
    return CanoFormWithAutGroupOfTable(TableByPowerBasis(A));
end;

InstallMethod( CanoFormWithAutGroup, [IsAlgebra], function( A )
    return CanoFormWithAutGroupOfTable(TableByPowerBasis(A));
end );

InstallMethod( CanonicalForm, [IsAlgebra], function(A)
    return CanoFormWithAutGroup(A).cano;
end );

InstallMethod( AutomorphismGroup, [IsAlgebra], function(A)
    local G, d, F, a, B;
    G := CanoFormWithAutGroup(A);
    d := Length(G.one);
    F := G.field;
    a := Concatenation(G.glAutos, G.agAutos);
    B := Subgroup(GL(d, F), a); 
    SetSize(B, G.size);
    return B;
end );



