
InstallMethod( ProductIdeal, [IsAlgebra, IsAlgebra], function(I,J)
    local K, b, c, k, i, j;
    K := Parent(I);
    if K <> Parent(J) then TryNextMethod(); fi;
    b := Basis(I);
    c := Basis(J);
    k := [];
    for i in [1..Length(b)] do
        for j in [1..Length(c)] do
            Add(k, b[i]*c[j]);
        od;
    od;
    return SubalgebraNC( K, k );
end );

InstallMethod( PowerSeries, [IsAlgebra], function(J)
    local s, I, K;
    s := [J];
    I := StructuralCopy(J);
    while Dimension(I) > 0 do 
        K := ProductIdeal( J, I );
        if Dimension(K) = Dimension(I) then return s; fi;
        I := K; Add( s, I );
    od;
    return s;
end );

InstallMethod( IsNilpotentAlgebra, [IsAlgebra], function(I)
    local s;
    s := PowerSeries(I);
    return Dimension(s[Length(s)]) = 0;
end );

InstallMethod( PowerBasis, [IsAlgebra], function(J)
    local s, b, i, nat, c, w;
    s := PowerSeries(J);
    b := [];
    w := [];
    for i in [1..Length(s)-1] do
        nat := NaturalHomomorphismByIdeal(s[i],s[i+1]);
        c := Basis(Image(nat));
        Append( b, List(c, x -> PreImagesRepresentative(nat,x)));
        Append( w, List(c, x -> i));
    od;
    return rec( basis := b, weights := w);
end );

