
FPMinOverIdeals := function(Tab, v)
    local F, w, T, n, I, d, m, V, U, i, j, res;

    # set up
    F := Tab.field;
    T := Tab.table;
    w := Tab.weights;
    d := Tab.rank;
    n := Length(T);
    I := IdentityMat(n, F);

    # get min overs 
    m := List(v, x -> x * I{[1..d]});

    # get products in I^2
    V := [];
    for i in [d+1..n] do
        for j in [d+1..n] do
            Add( V, T[i][j] );
        od;
    od;
    V := MyBaseMat(V);

    # add products with min overs
    res := [];
    for i in [1..Length(m)] do
        U := StructuralCopy(V);
        Add(U, MultBySC( T, F, m[i], m[i] ));
        for j in [d+1..n] do
            Add(U, MultBySC( T, F, m[i], I[j] ));
        od;
        for j in [d+1..n] do
            Add(U, MultBySC( T, F, I[j], m[i] ));
        od;
        res[i] := Length(MyBaseMat(U));
    od;

    # that's it
    return res;
end;

InitAutomGroup := function( Tab )
    local d, F, H, G, V, v, m, s, inv, ind, bas, h, P, i, l, w, n, I;

    # set up full GL
    d := Tab.rank;
    F := Tab.field;
    H := GL(d, F);
    V := F^d;
    n := Length(Tab.table);
    I := IdentityMat(n,F);

    # set up aut group record
    G := rec();
    G.agAutos := [];
    G.one     := One(H);
    G.field   := F;

    # use subgroup of H if desired
    if USE_PARTI then 

        # get min over spaces
        v := List(NormedRowVectors(V), Reversed);

        # choose further option
        if IsInt(USE_FULL) then 
            w := List(v, x -> Concatenation([x*I{[1..d]}], I{[d+1..n]}));
            m := [];
            for i in [1..Length(w)] do
                m[i] := Collected(PowerBasisFunc(Tab,w[i],2,USE_FULL).weights);
                Info( InfoModIsom, 1, "   found weights ",m[i]);
            od;
        else
            m := FPMinOverIdeals(Tab, v);
        fi;

        s := Set(m);
        Info( InfoModIsom, 1, "   found partition ",Collected(m));

        # translate to invariant subspaces
        inv := [1..Length(s)];
        for i in [1..Length(s)] do
            ind := Filtered([1..Length(m)], x -> m[x] = s[i]);
            inv[i] := MyBaseMat( v{ind} );
        od;

        # create a chain from invariant subspaces
        inv := Filtered(inv, x -> Length(x)<d);
        bas := BasisBySubspaces( inv, F, d );
        Info( InfoModIsom, 1, "   found basis with weights ",bas.weights);

        # compute mat group that stabilizes this chain
        H := ChainStabilizer( bas.weights, F );

        # reset and add relevant entries in aut group record
        G.basis   := bas.basis;
        G.partition := Collected(m);
    elif USE_CHARS then 
        
        # get char subs
        inv := TwoStepCents( Tab );
        bas := BasisBySubspaces( inv, F, d );
        Info( InfoModIsom, 1, "   found basis with weights ",bas.weights);

        # compute mat group that stabilizes this chain
        H := ChainStabilizer( bas.weights, F );

        # reset and add relevant entries in aut group record
        G.basis   := bas.basis;
        G.partition := false;
    else
        G.basis   := IdentityMat(d,F);
        G.partition := false;
    fi;

    # add perm action 
    v := Filtered( Elements(V), x -> x <> Zero(V) );
    h := ActionHomomorphism( H, v, OnRight, "surjective" );
    P := Image(h);
    SetSize(P, Size(H));

    # reset relevant entries in aut group record
    G.glPerms := SmallGeneratingSet(P);
    G.glAutos := List(G.glPerms, x -> PreImagesRepresentative(h,x));
    G.glOrder := Size(H);
    G.size    := Size(H);

    # that's it
    return G;
end;

