
CleanBasis := function( bas, wgt, d )
    local i;
    for i in [1..d-1] do
        if IsBound(bas[i]) and wgt[i] < wgt[d] then 
            AddRowVector( bas[i], bas[d], bas[i][d] );
        fi;
    od;
end;

SiftIntoBasis := function( bas, wgt, rel, b, w )
    local d, n, c, v, a;
    d := DepthVector(b);
    n := Length(b);
    while d <= n do
        if IsBool(wgt[d]) then 
            bas[d] := b;
            wgt[d] := w;
            rel[d] := true;
            CleanBasis(bas, wgt, d);
            return;
        elif wgt[d] >= w then
            AddRowVector( b, bas[d], - b[d]);
            d := DepthVector(b);
        elif wgt[d] < w then 
            c := bas[d]; bas[d] := b[d]^-1 * b; b := bas[d] - c;
            v := wgt[d]; wgt[d] := w; w := v;
            rel[d] := true;
            d := DepthVector(b);
        fi;
    od;
end;

PowerBasisFunc := function( Tab, U, cov, lim )
    local n, bas, wgt, rel, u, i, j, b, w, a;

    # set up
    n := Length(U[1]);
    bas := [];
    wgt := List([1..n], x -> false);
    rel := List([1..n], x -> true);

    # sift elements of U in
    for u in U do
        w := Tab.weights[DepthVector(u)];
        if w < cov then 
            SiftIntoBasis( bas, wgt, rel, u, 1 );
        else
            SiftIntoBasis( bas, wgt, rel, u, Minimum(QuoInt(w,cov),lim) );
        fi;
    od;

    # adjust relevance
    for i in [Position(Tab.weights, cov)..n] do
        rel[i] := false;
    od;

    # loop
    while ForAny(rel, x -> x = true) do
        for i in [1..n] do
            if IsBound(bas[i]) and rel[i]=true then
                for j in [1..n] do
                    if IsBound(bas[j]) 
                       and (wgt[i]=1 or wgt[j]=1) 
                       and (wgt[i]+wgt[j]<=lim) 
                    then
                        b := MultBySC( Tab.table, Tab.field, bas[i], bas[j] );
                        SiftIntoBasis( bas, wgt, rel, b, wgt[i]+wgt[j] );
                        b := MultBySC( Tab.table, Tab.field, bas[j], bas[i] );
                        SiftIntoBasis( bas, wgt, rel, b, wgt[i]+wgt[j] );
                    fi;
                od;
            fi;
            rel[i] := false;
        od;
    od;

    bas := Compacted(bas);
    wgt := Filtered(wgt, x -> not IsBool(x));
    SortParallel(wgt, bas);
    return rec( basis := bas, weights := wgt );
end;

CentSubspaceMod := function(Tab, U, m)
    local T, S, n, M, i, k, new, j, N;

    # set up
    T := Tab.table;
    n := Length(T);
    M := List([1..m], x -> []);

    # precomp
    S := List([1..m], x -> List([1..m], y -> 0));
    for i in [1..m] do
        for j in [i+1..m] do
            S[i][j] := T[i][j]{[1..m]} - T[j][i]{[1..m]};
        od;
    od;

    # construct matrix
    for i in [1..Length(U)] do
        for k in [1..m] do
            new := 0*S[1][2];
            for j in [1..m] do
                if k > j then
                    new := new + U[i][j]*S[j][k];
                elif k < j then
                    new := new - U[i][j]*S[k][j];
                fi;
            od;
            Append(M[k], new);
        od;
    od;

    # compute nullspace
    N := TriangulizedNullspaceMat( M );
    return N;
end;

TwoStepCents := function(Tab)
    local F, T, w, s, d, res, k, f, m, l, M, i, j, N;

    # set up
    F := Tab.field;
    T := Tab.table;
    w := Tab.weights;
    s := w[Length(w)];
    d := Tab.rank;

    res := [];
    for k in [1..s-2] do
        f := Position(w, k);
        m := Position(w, k+1);
        l := Position(w, k+2);
        if l = fail then l := Length(w)+1; fi;

        M := List([1..l-1], x -> []);
        for i in [f..m-1] do
            for j in [1..l-1] do
                Append(M[j], T[i][j]{[1..l-1]} - T[j][i]{[1..l-1]} );
            od;
        od;

        N := NullspaceMat(M * One(F));
        N := List(N, x -> x{[1..d]});
        N := MyBaseMat(N);
        if Length(N)>0 and Length(N)<d then Add(res, N); fi;
    od;

    return res;
end;

