VecToList := function(vec)
    local l, i;
    l := [];
    for i in [1..Length(vec)] do
        if vec[i] <> 0*vec[i] then
            Add( l, vec[i] );
            Add( l, i );
        fi;
    od;
    return l;
end;

InstallMethod( AlgebraByTable, [IsObject], function( T )
    local n, F, S, i, j;
    n := Length(T);
    F := Field(T[1][1][1]);
    S := EmptySCTable( n, Zero(F) );
    for i in [1..n] do
        for j in [1..n] do
            SetEntrySCTable( S, i, j, VecToList(T[i][j]));
        od;
    od;
    return AlgebraByStructureConstants(F, S);
end);

InstallMethod( TableByBasis, [IsAlgebra, IsList], function( A, B )
    local b, a, n, F, S, i, j, c;
    b := Basis(A);
    a := List(B, x -> Coefficients(b,x));
    n := Length(B);
    F := LeftActingDomain(A);
    S := List( [1..n], x -> MutableNullMat(n,n,F));
    for i in [1..n] do
        for j in [1..n] do
            c := Coefficients(b, B[i]*B[j]);
            S[i][j] := SolutionMat(a,c);
        od;
    od;
    return S;
end );

InstallMethod( TableByPowerBasis, [IsAlgebra], function( A )
    local b, T;
    b := PowerBasis(A);
    T := TableByBasis(A, b.basis); 
    return rec( table := T, 
                weights := b.weights,
                field := LeftActingDomain(A),
                rank := Length(Filtered(b.weights, x -> x=1)) );
end );

IsNewBasisVector := function( B, c )
    if c = 0*c then return false; fi;
    if Length(B) = 0 then return true; fi;
    return IsBool(SolutionMat(B, c));
end;

MultBySC := function( S, F, v, w )
    local u, i, j;
    u := ShallowCopy( 0 * v );
    ConvertToVectorRep( u, F );
    for i in [1..Length(v)] do
        if v[i] <> Zero(F) then
            for j in [1..Length(w)] do
                if w[j] <> Zero(F) then
                    AddRowVector( u, S[i][j], v[i]*w[j] );
                fi;
            od;
        fi;
    od;
    return u;
end;

